Module: Subversion

Defined in:
lib/subwrap/subversion.rb,
lib/subwrap/subversion.rb,
lib/subwrap/subversion_extensions.rb

Overview

These are methods used by the SvnCommand for filtering and whatever else it needs… It could probably be moved into SvnCommand, but I thought it might be good to at least make it possible to use them apart from SvnCommand. Rename to Subversion::Filters ? Then each_unadded would be an odd man out.

Defined Under Namespace

Modules: Extensions Classes: Diff, Diffs, DiffsParser, ExternalsContainer, RevisionProperty

Constant Summary collapse

@@color =

True if you want output from svn to be colorized (useful if output is for human eyes, but not useful if using the output programatically)

false
@@dry_run =

If true, will only output which command would have been executed but will not actually execute it.

false
false

Class Method Summary collapse

Class Method Details

.add(*args) ⇒ Object

Adds the given items to the repository. Items may contain wildcards.



56
57
58
# File 'lib/subwrap/subversion.rb', line 56

def self.add(*args)
  execute "add #{args.join ' '}"
end

.add_to_property(property, path, *new_lines) ⇒ Object

It’s easy to get/set properties, but less easy to add to a property. This method uses get/set to simulate add. It will uniquify lines, removing duplicates. (:todo: what if we want to set a property to have some duplicate lines?)



234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
# File 'lib/subwrap/subversion.rb', line 234

def self.add_to_property(property, path, *new_lines)
  # :todo: I think it's possible to have properties other than svn:* ... so if property contains a prefix (something:), use it; else default to 'svn:'
  
  # Get the current properties
  lines = self.get_property(property, path).split "\n"
  puts "Existing lines: #{lines.inspect}" if $debug

  # Add the new lines, delete empty lines, and uniqueify all elements
  lines.concat(new_lines).uniq!
  puts "After concat(new_lines).uniq!: #{lines.inspect}" if $debug

  lines.delete ''
  # Set the property
  puts "About to set propety to: #{lines.inspect}" if $debug
  self.set_property property, lines.join("\n"), path
end

.base_url(path_or_url = './') ⇒ Object

:todo: needs some serious unit-testing love



345
346
347
348
349
350
351
352
353
354
355
356
357
358
359
360
361
362
363
364
365
366
367
368
369
# File 'lib/subwrap/subversion.rb', line 345

def self.base_url(path_or_url = './')
  matches = info(path_or_url).match(/^Repository Root: (.+)/)
  matches && matches[1]

   # It appears that we might need to use this old way (which looks at 'URL'), since there is actually a handy property called "Repository Root" that we can look at.
#    base_url = nil    # needed so that base_url variable isn't local to loop block (and reset during next iteration)!
#    started_using_dot_dots = false
#    loop do
#      matches = /^URL: (.+)/.match(info(path_or_url))
#      if matches && matches[1]
#        base_url = matches[1]
#      else
#        break base_url
#      end
#
#      # Keep going up the path, one directory at a time, until `svn info` no longer returns a URL (will probably eventually return 'svn: PROPFIND request failed')
#      if path_or_url.include?('/') && !started_using_dot_dots
#        path_or_url = File.dirname(path_or_url)
#      else
#        started_using_dot_dots = true
#        path_or_url = File.join(path_or_url, '..')
#      end
#      #puts 'going up to ' + path_or_url
#    end
end

.cat(*args) ⇒ Object



227
228
229
230
# File 'lib/subwrap/subversion.rb', line 227

def self.cat(*args)
  args = ['./'] if args.empty?
  execute("cat #{args.join ' '}")
end

.commit(*args) ⇒ Object



170
171
172
173
# File 'lib/subwrap/subversion.rb', line 170

def self.commit(*args)
  args = ['./'] if args.empty?
  execute("commit #{args.join ' '}")
end

.delete_property(property, path = './') ⇒ Object



259
260
261
# File 'lib/subwrap/subversion.rb', line 259

def self.delete_property(property, path = './')
  execute "propdel svn:#{property} #{path}"
end

.delete_revision_property(property_name, rev) ⇒ Object



262
263
264
# File 'lib/subwrap/subversion.rb', line 262

def self.delete_revision_property(property_name, rev)
  execute("propdel --revprop #{property_name} -r #{rev}").chomp
end

.diff(*args) ⇒ Object

Returns the modifications to the working directory or URL specified in args.



213
214
215
216
# File 'lib/subwrap/subversion.rb', line 213

def self.diff(*args)
  args = ['./'] if args.empty?
  execute("diff #{"--diff-cmd colordiff" if color?} #{args.join ' '}")
end

.diffs(*args) ⇒ Object

Parses the output from diff and returns an array of Diff objects.



218
219
220
221
222
223
224
225
# File 'lib/subwrap/subversion.rb', line 218

def self.diffs(*args)
  args = ['./'] if args.empty?
  raw_diffs = nil
  with_color! false do
    raw_diffs = diff(*args)
  end
  DiffsParser.new(raw_diffs).parse
end

.executableObject

The location of the executable to be used to do: Is there a smarter/faster way to do this? (Could cache this result in .subwrap or somewhere, so we don’t have to do all this work on every invocation…)



408
409
410
411
412
413
414
415
416
417
418
# File 'lib/subwrap/subversion.rb', line 408

def self.executable
  # FileUtils.which('svn') would return our Ruby *wrapper* script for svn. We actually want to return here the binary/executable that we are
  # *wrapping* so we have to use whereis and then use the first one that is ''not'' a Ruby script.
  @@executable ||=
    FileUtils.whereis('svn') do |executable|
      if !self.ruby_script?(executable)               # We want to wrap the svn binary provided by Subversion, not our custom replacement for that.
        return windows_platform? ? %{"#{executable}"} : executable
      end
    end
  raise 'svn binary not found'
end

.export(path_or_url, target) ⇒ Object



101
102
103
# File 'lib/subwrap/subversion.rb', line 101

def self.export(path_or_url, target)
  execute "export #{path_or_url} #{target}"
end

.externalize(repo_url, options = {}) ⇒ Object

Adds the given repository URL (svn.yourcompany.com/path/to/something) as an svn:externals.

Options may include:

  • :as - overrides the default behavior of naming the checkout based on the last component of the repo path

  • :local_path - specifies where to set the externals property. Defaults to ‘.’ or the dirname of as if as is specified (for example, vendor/plugins if as is vendor/plugins/plugin_name).



86
87
88
89
90
91
92
93
94
95
96
97
98
99
# File 'lib/subwrap/subversion.rb', line 86

def self.externalize(repo_url, options = {})

  options[:as] ||= File.basename(repo_url)
  #options[:as] = options[:as].ljust(29)

  # You can't set the externals of './' to 'vendor/plugins/foo http://example.com/foo'
  # Instead, you have to set the externals of 'vendor/plugins/' to 'foo http://example.com/foo'
  # This will make that correction for you automatically.
  options[:local_path] ||= File.dirname(options[:as])   # Will be '.' if options[:as] has no dirname component.
                                                        # Will be 'vendor/plugins' if options[:as] is 'vendor/plugins/plugin_name'.
  options[:as] = File.basename(options[:as])

  add_to_property 'externals', options[:local_path], "#{options[:as]} #{repo_url}"
end

.externals_containers(path = './') ⇒ Object

Returns an array of ExternalsContainer objects representing all externals containers in the working directory specified by path.



201
202
203
204
205
206
207
208
209
210
# File 'lib/subwrap/subversion.rb', line 201

def self.externals_containers(path = './')
  # Using self.externals_items is kind of a cheap way to do this, and it results in some redundancy that we have to filter out
  # (using uniq_by), but it seemed more efficient than the alternative (traversing the entire directory tree and querying for
  # `svn prepget svn:externals` at each stop to see if the directory is an externals container).
  self.externals_items(path).map { |external_dir|
    ExternalsContainer.new(external_dir + '/..')
  }.uniq_by { |external|
    external.container_dir
  }
end

.externals_items(path = './') ⇒ Object

Returns an array of externals items. These are the actual externals listed in an svn:externals property. Example:

vendor/a
vendor/b

Where ‘vendor’ is an ExternalsContainer containing external items ‘a’ and ‘b’.



188
189
190
191
192
193
194
195
196
197
198
# File 'lib/subwrap/subversion.rb', line 188

def self.externals_items(path = './')
  status = status_the_section_before_externals(path)
  return [] if status.nil?
  status.select { |line|
    line =~ /^X/
  }.map { |line|
    # Just keep the filename part
    line =~ /^X\s+(.+)/
    $1
  }
end

.get_property(property, path = './') ⇒ Object

:todo: Stop assuming the svn: namespace. What’s the point of a namespace if you only allow one of them?



252
253
254
# File 'lib/subwrap/subversion.rb', line 252

def self.get_property(property, path = './')
  execute "propget svn:#{property} #{path}"
end

.get_revision_property(property_name, rev) ⇒ Object



255
256
257
# File 'lib/subwrap/subversion.rb', line 255

def self.get_revision_property(property_name, rev)
  execute("propget --revprop #{property_name} -r #{rev}").chomp
end

.help(*args) ⇒ Object



297
298
299
# File 'lib/subwrap/subversion.rb', line 297

def self.help(*args)
  execute "help #{args.join(' ')}"
end

.ignore(*patterns) ⇒ Object

Sets the svn:ignore property based on the given patterns. Each pattern is both the path (where the property gets set) and the property itself. For instance:

"log/*.log" would add "*.log" to the svn:ignore property on the log/ directory.
"log" would add "log" to the svn:ignore property on the ./ directory.


65
66
67
68
69
70
71
72
73
74
# File 'lib/subwrap/subversion.rb', line 65

def self.ignore(*patterns)
  
  patterns.each do |pattern|
    path = File.dirname(pattern)
    path += '/' if path == '.'
    pattern = File.basename(pattern)
    add_to_property 'ignore', path, pattern
  end
  nil
end

.info(*args) ⇒ Object



334
335
336
337
# File 'lib/subwrap/subversion.rb', line 334

def self.info(*args)
  args = ['./'] if args.empty?
  execute "info #{args.join(' ')}"
end

.latest_revision(*args) ⇒ Object

Returns the revision number for head.



307
308
309
310
311
312
# File 'lib/subwrap/subversion.rb', line 307

def self.latest_revision(*args)
  args = ['./'] if args.empty?
  # The revision returned by svn status -u seems to be a pretty reliable way to get this. Does anyone know of a better way?
  matches = /Status against revision:\s+(\d+)/m.match(status_against_server(args))
  matches && matches[1]
end

.latest_revision_for_path(path) ⇒ Object

Returns the revision number for the working directory(/file?) specified by path



314
315
316
317
318
# File 'lib/subwrap/subversion.rb', line 314

def self.latest_revision_for_path(path)
  # The revision returned by svn info seems to be a pretty reliable way to get this. Does anyone know of a better way?
  matches = info(path).match(/^Revision: (\d+)/)
  matches && matches[1]
end

.log(*args) ⇒ Object

Returns the raw output from svn log



302
303
304
305
# File 'lib/subwrap/subversion.rb', line 302

def self.log(*args)
  args = ['./'] if args.empty?
  execute "log #{args.join(' ')}"
end

.make_directory(dir) ⇒ Object



293
294
295
# File 'lib/subwrap/subversion.rb', line 293

def self.make_directory(dir)
  execute "mkdir #{dir}"
end

.make_executable(*paths) ⇒ Object

Marks the given items as being executable. Items may not contain wildcards.



143
144
145
146
147
# File 'lib/subwrap/subversion.rb', line 143

def self.make_executable(*paths)
  paths.each do |path|
    self.set_property 'executable', '', path
  end
end

.make_not_executable(*paths) ⇒ Object



148
149
150
151
152
# File 'lib/subwrap/subversion.rb', line 148

def self.make_not_executable(*paths)
  paths.each do |path|
    self.delete_property 'executable', path
  end
end

.proplist(rev) ⇒ Object

Gets raw output of proplist command



274
275
276
# File 'lib/subwrap/subversion.rb', line 274

def self.proplist(rev)
  execute("proplist --revprop -r #{rev}")
end

.remove(*args) ⇒ Object

Removes the given items from the repository and the disk. Items may contain wildcards.



106
107
108
# File 'lib/subwrap/subversion.rb', line 106

def self.remove(*args)
  execute "rm #{args.join ' '}"
end

.remove_force(*args) ⇒ Object

Removes the given items from the repository and the disk. Items may contain wildcards. To do: add a :force => true option to remove



112
113
114
# File 'lib/subwrap/subversion.rb', line 112

def self.remove_force(*args)
  execute "rm --force #{args.join ' '}"
end

.remove_without_delete(*args) ⇒ Object

Removes the given items from the repository BUT NOT THE DISK. Items may contain wildcards.



117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
# File 'lib/subwrap/subversion.rb', line 117

def self.remove_without_delete(*args)
  # resolve the wildcards before iterating
  args.collect {|path| Dir[path]}.flatten.each do |path|
    entries_file = "#{File.dirname(path)}/.svn/entries"
    File.chmod(0644, entries_file)

    xmldoc = REXML::Document.new(IO.read(entries_file))
    # first attempt to delete a matching entry with schedule == add
    unless xmldoc.root.elements.delete "//entry[@name='#{File.basename(path)}'][@schedule='add']"
      # then attempt to alter a missing schedule to schedule=delete
      entry = REXML::XPath.first(xmldoc, "//entry[@name='#{File.basename(path)}']")
      entry.attributes['schedule'] ||= 'delete' if entry
    end
    # write back to the file
    File.open(entries_file, 'w') { |f| xmldoc.write f, 0 }

    File.chmod(0444, entries_file)
  end
end

.repository_root(*args) ⇒ Object



371
# File 'lib/subwrap/subversion.rb', line 371

def self.repository_root(*args); base_url(*args); end

.repository_uuid(path_or_url = './') ⇒ Object



374
375
376
377
# File 'lib/subwrap/subversion.rb', line 374

def self.repository_uuid(path_or_url = './')
  matches = info(path_or_url).match(/^Repository UUID: (.+)/)
  matches && matches[1]
end

.revert(*args) ⇒ Object

Reverts the given items in the working copy. Items may contain wildcards.



138
139
140
# File 'lib/subwrap/subversion.rb', line 138

def self.revert(*args)
  execute "revert #{args.join ' '}"
end

.revision_properties(rev) ⇒ Object

Returns an array of RevisionProperty objects (name, value) for revisions currently set on the given rev Tessted by: ../../test/subversion_test.rb:test_revision_properties



287
288
289
290
291
# File 'lib/subwrap/subversion.rb', line 287

def self.revision_properties(rev)
  revision_properties_names(rev).map { |property_name|
    RevisionProperty.new(property_name, get_revision_property(property_name, rev))
  }
end

.revision_properties_names(rev) ⇒ Object

Returns an array of the names of all revision properties currently set on the given rev Tessted by: ../../test/subversion_test.rb:test_revision_properties_names



279
280
281
282
283
284
# File 'lib/subwrap/subversion.rb', line 279

def self.revision_properties_names(rev)
  raw_list = proplist(rev)
  raw_list.scan(/^ +([^ ]+)$/).map { |matches|
    matches.first.chomp
  }
end

.revisions(*args) ⇒ Object

Returns an array of RSCM::Revision objects



321
322
323
324
325
326
327
328
329
330
331
# File 'lib/subwrap/subversion.rb', line 321

def self.revisions(*args)
  # Tried using this, but it seems to expect you to pass in a starting date or accept the default starting date of right now, which is silly if you actually just want *all* revisions...
  #@rscm = ::RSCM::Subversion.new
  #@rscm.revisions

  #log_output = Subversion.log('-v')
  log_output = Subversion.log(*(['-v'] + args))
  parser = ::RSCM::SubversionLogParser.new(io = StringIO.new(log_output), url = 'http://ignore.me.com')
  revisions = parser.parse_revisions
  revisions
end

.root_url(*args) ⇒ Object

base_url = nil # needed so that base_url variable isn’t local to loop block (and reset during next iteration)!

started_using_dot_dots = false
loop do
  matches = /^URL: (.+)/.match(info(path_or_url))
  if matches && matches[1]
    base_url = matches[1]
  else
    break base_url
  end

  # Keep going up the path, one directory at a time, until `svn info` no longer returns a URL (will probably eventually return 'svn: PROPFIND request failed')
  if path_or_url.include?('/') && !started_using_dot_dots
    path_or_url = File.dirname(path_or_url)
  else
    started_using_dot_dots = true
    path_or_url = File.join(path_or_url, '..')
  end
  #puts 'going up to ' + path_or_url
end


370
# File 'lib/subwrap/subversion.rb', line 370

def self.root_url(*args);        base_url(*args); end

.ruby_script?(file_path) ⇒ Boolean

Returns:

  • (Boolean)


420
421
422
423
424
425
426
427
# File 'lib/subwrap/subversion.rb', line 420

def self.ruby_script?(file_path)
  if windows_platform?
    # The 'file' command, we assume, is not available
    File.readlines(file_path)[0] =~ /ruby/
  else
    `file #{file_path}` =~ /ruby/
  end
end

.set_property(property, value, path = './') ⇒ Object



266
267
268
# File 'lib/subwrap/subversion.rb', line 266

def self.set_property(property, value, path = './')
  execute "propset svn:#{property} '#{value}' #{path}"
end

.set_revision_property(property_name, rev) ⇒ Object



269
270
271
# File 'lib/subwrap/subversion.rb', line 269

def self.set_revision_property(property_name, rev)
  execute("propset --revprop #{property_name} -r #{rev}").chomp
end

.status(*args) ⇒ Object

Returns the status of items in the working directories paths. Returns the raw output from svn (use split("\n") if you want an array).



155
156
157
158
# File 'lib/subwrap/subversion.rb', line 155

def self.status(*args)
  args = ['./'] if args.empty?
  execute("status #{args.join ' '}")
end

.status_against_server(*args) ⇒ Object



160
161
162
163
# File 'lib/subwrap/subversion.rb', line 160

def self.status_against_server(*args)
  args = ['./'] if args.empty?
  self.status('-u', *args)
end

.status_the_section_before_externals(path = './') ⇒ Object

The output from ‘svn status` is nicely divided into two “sections”: the section which pertains to the current working copy (not counting externals as part of the working copy) and then the section with status of all of the externals. This method returns the first section.



178
179
180
181
# File 'lib/subwrap/subversion.rb', line 178

def self.status_the_section_before_externals(path = './')
  status = status(path) || ''
  status.sub!(/(Performing status.*)/m, '')
end

.under_version_control?(file = './', strict = false) ⇒ Boolean

By default, if you query a directory that is scheduled for addition but hasn’t been committed yet (node doesn’t have a UUID), then we will still return true, because it is scheduled to be under version control. If you want a stricter definition, and only want it to return true if the file exists in the repository (has a UUID)@ then pass strict = true

Returns:

  • (Boolean)


382
383
384
385
386
387
388
# File 'lib/subwrap/subversion.rb', line 382

def self.under_version_control?(file = './', strict = false)
  if strict
    !!repository_uuid(file)
  else # (scheduled_for_addition_counts_as_true)
    !!url(file)
  end
end

.unignore(*patterns) ⇒ Object

Raises:

  • (NotImplementedError)


75
76
77
# File 'lib/subwrap/subversion.rb', line 75

def self.unignore(*patterns)
  raise NotImplementedError
end

.update(*args) ⇒ Object



165
166
167
168
# File 'lib/subwrap/subversion.rb', line 165

def self.update(*args)
  args = ['./'] if args.empty?
  execute("update #{args.join ' '}")
end

.url(path_or_url = './') ⇒ Object



339
340
341
342
# File 'lib/subwrap/subversion.rb', line 339

def self.url(path_or_url = './')
  matches = info(path_or_url).match(/^URL: (.+)/)
  matches && matches[1]
end

.working_copy_root(directory = './') ⇒ Object



389
390
391
392
393
394
395
396
397
398
399
400
401
402
403
404
# File 'lib/subwrap/subversion.rb', line 389

def self.working_copy_root(directory = './')
  uuid = repository_uuid(directory)
  return nil if uuid.nil?

  loop do
    # Keep going up, one level at a time, ...
    new_directory = File.expand_path(File.join(directory, '..'))
    new_uuid = repository_uuid(new_directory)

    # Until we get back a uuid that is nil (it's not a working copy at all) or different (you can have a working copy A inside of a different WC B)...
    break if new_uuid.nil? or new_uuid != uuid

    directory = new_directory
  end
  directory
end